<?php
/* --------------------------------------------------------------
   LexofficeInvoiceModuleController.inc.php 2020-09-11
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

class LexofficeInvoiceModuleController extends AdminHttpViewController
{
    /**
     * lexoffice API Service
     * @var LexofficeInvoiceService
     */
    protected $lexofficeService;

    /**
     * @var LanguageTextManager $languageTextManager
     */
    protected $languageTextManager;

    /**
     * @var LexofficeInvoiceLogger $logger
     */
    protected $logger;

    protected function init()
    {
        $this->configuration = MainFactory::create('LexofficeInvoiceConfigurationStorage');
        $this->languageTextManager = MainFactory::create('LanguageTextManager', 'module_center_module');
        $this->lexofficeService = MainFactory::create('LexofficeInvoiceService', $this->configuration);
        $this->logger = MainFactory::create('LexofficeInvoiceLogger');
    }


    public function actionCreateInvoice()
    {
        require_once(DIR_FS_INC . 'xtc_get_attributes_model.inc.php');
        require_once(DIR_FS_INC . 'xtc_format_price_order.inc.php');
        require_once(DIR_FS_CATALOG . 'gm/inc/gm_prepare_number.inc.php');
        require_once(DIR_WS_CLASSES . 'order.php');

        $gmFormat = new GMOrderFormat();

        /** @var InvoiceArchiveReadService $invoiceArchiveReadService */
        $invoiceArchiveReadService = StaticGXCoreLoader::getService('InvoiceArchiveRead');

        // set received order id
        $orderId = $_GET['oID'];

        // check if invoice is of type storno/creditnote
        $cancelInvoice = array_key_exists('cancel_invoice_id', $_GET);

        /**
         * -> create order
         */
        $order = new order($orderId);

        /**
         * -> get order data
         */
        $order_check = $this->getOrderCheckData();
        $order_data = $this->getOrderData();

        /**
         * check if a new invoice pdf has to be created
         * do not create on send email for existing invoice
         */
        $createPdf = $_GET['mail'] !== '1'
            || ((!array_key_exists('create_missing_invoices', $_POST)
                || (array_key_exists('create_missing_invoices', $_POST)
                    && $_POST['create_missing_invoices'] === '1'))
                && (!array_key_exists('invoice_ids', $_POST) || empty($_POST['invoice_ids']))
                && (empty($_GET['iID'])));

        $redirect = false;
        $createdInvoiceIds = [];

        if ($createPdf) {
            try {
                /**
                 * -> order total data
                 */
                $order_total = $this->getOrderTotalData();

                /**
                 * -> order payment instructions
                 */
                $order_payment_instruction = $this->getOrderPaymentInstruction($order->info['payment_method']);

                // -> invoice or packings billing code?
                if ($_GET['type'] === 'invoice') //ajax=1
                {
                    /**
                     * invoice code
                     */

                    $t_invoice_date = new DateTime();
                    /* determine invoice date */
                    $t_invoice_date = new DateTime(gm_get_conf('GM_PDF_INVOICE_DATE'));

                    /*
                     * Create lexoffice invoice
                     */

                    try {

                        // read lexoffice profile
                        $lexofficeProfile = (array)$this->lexofficeService->readLexofficeProfile();
                        // auto update taxType from profile
                        $this->configuration->setTaxType($lexofficeProfile['taxType']);
                        $this->configuration->setSmallBusiness($lexofficeProfile['smallBusiness'] ? '1' : '0');

                        if ($cancelInvoice) {
                            $lxoCreditNoteRequest = $this->prepareLexofficeCreditNoteData($order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction);
                            $lxoCreatedCreditNote = $this->lexofficeService->createLexofficeCreditNote($lxoCreditNoteRequest);

                            if ($lxoCreatedCreditNote) {
                                $lxoVoucherResponse = $this->lexofficeService->readLexofficeCreditNote($lxoCreatedCreditNote->id);
                                $lxoCreditNoteFileResponse = $this->lexofficeService->renderLexofficeCreditNotePdf($lxoCreatedCreditNote->id);
                            }
                            if ($lxoCreditNoteFileResponse) {
                                $lxoInvoiceFile = $this->lexofficeService->readLexofficeFile($lxoCreditNoteFileResponse->documentFileId);
                            }
                        } else {
                            $lxoInvoiceRequest = $this->prepareLexofficeInvoiceData($order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction);
                            $lxoCreatedInvoice = $this->lexofficeService->createLexofficeInvoice($lxoInvoiceRequest);

                            if ($lxoCreatedInvoice) {
                                $lxoVoucherResponse = $this->lexofficeService->readLexofficeInvoice($lxoCreatedInvoice->id);
                                $lxoInvoiceFileResponse = $this->lexofficeService->renderLexofficeInvoicePdf($lxoCreatedInvoice->id);
                            }
                            if ($lxoInvoiceFileResponse) {
                                $lxoInvoiceFile = $this->lexofficeService->readLexofficeFile($lxoInvoiceFileResponse->documentFileId);
                            }
                        }
                    } catch (LexofficeInvoiceException | LexofficeInvoiceAuthException $e) {
                        $this->logger->error('Error creating lexoffice invoice: ' . $e);
                        $title    = new NonEmptyStringType($this->languageTextManager->get_text('lexoffice_invoice_api_error'));
                        $template = $this->getTemplateFile('Agiltech/LexofficeInvoice/Admin/Html/lexoffice_invoice_error.html');

                        $data = MainFactory::create(
                            'KeyValueCollection',
                            [
                                'error'            => $e->getMessage(),
                                'details'          => $e->getDetails(),
                            ]
                        );
                        return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data);
                    }

                    if (($lxoCreatedInvoice || $lxoCreatedCreditNote) && $lxoInvoiceFile) {
                        /**
                         * -> change order status only when create an invoice
                         */
                        $this->changeOrderStatus($gmFormat, $order_check, $cancelInvoice);

                        // set new created lxo voucher number
                        $order_check['gm_orders_code'] = $lxoVoucherResponse->voucherNumber;
                        // do not set "_STORNO" idetifier if cancel invoice - this is currently the only mechanism to identify the invoice as cancelled!

                        // pdf file name
                        $t_pdf_name = xtc_cleanName(trim($gm_order_pdf_values['GM_PDF_HEADING']), '_') . '_' . $next_id . '.pdf';
                        $t_pdf_path = DIR_FS_CATALOG . 'export/invoice/' . (int)$orderId . '__' . str_replace('/', '_', $order_check['gm_orders_code']) . '__' . date("Y-m-d-H-i-s") . '__' . FileLog::get_secure_token() . '.pdf';

                        /**
                         * -> handle output -> write file to disk
                         */
                        file_put_contents($t_pdf_path, $lxoInvoiceFile);

                        /**
                         * The code below stores the invoice in the database. Most of the invoice data is fetched from the
                         * order entity which is provided by the order read service and the oID GET argument. (Braces are used
                         * for readability).
                         *
                         * @var OrderReadService $orderReadService
                         */
                        if ((int)$orderId !== 0) {
                            $orderReadService = StaticGXCoreLoader::getService('OrderRead');
                            $orderId          = new IdType((int)$orderId);
                            $order            = $orderReadService->getOrderById($orderId);

                            $invoiceFile   = MainFactory::create('ExistingFile', new NonEmptyStringType($t_pdf_path));
                            $invoiceNumber = new StringType($order_check['gm_orders_code']);
                            $invoiceDate = $t_invoice_date;

                            $currency      = $order->getCurrencyCode();

                            $orderTotalCollection = $order->getOrderTotals();
                            $totalSum             = null;
                            foreach ($orderTotalCollection as $orderTotal) {
                                /** @var OrderTotal $orderTotal */
                                if ($orderTotal->getClass() === 'ot_total') {
                                    $totalSum = $orderTotal->getValue();
                                    $totalSum = new DecimalType($totalSum);
                                }
                            }

                            $customerId                = new IdType($order->getCustomerId());
                            $customerStatusInformation = $order->getCustomerStatusInformation();
                            $paymentAddress            = $order->getBillingAddress();
                            $shippingAddress           = $order->getDeliveryAddress();
                            $orderPurchaseDate         = $order->getPurchaseDateTime();
                            $paymentType               = $order->getPaymentType();
                        } else if ($cancelInvoice) {
                            /**
                             * The code below stores the invoice in the database. Most of the invoice data is fetched from the
                             * order entity which is provided by the order read service and the oID GET argument. (Braces are used
                             * for readability).
                             *
                             * @var InvoiceArchiveReadService $invoiceArchiveReadService
                             */
                            $invoiceArchiveReadService = StaticGXCoreLoader::getService('InvoiceArchiveRead');
                            $orderId                   = new IdType((int)$orderId);
                            $originalInvoiceId         = new IdType((int)$_GET['cancel_invoice_id']);
                            $originalInvoice           = $invoiceArchiveReadService->getInvoiceListItemById($originalInvoiceId);

                            $invoiceFile   = MainFactory::create('ExistingFile', new NonEmptyStringType($t_pdf_path));
                            $invoiceNumber = new StringType($order_check['gm_orders_code']);
                            $invoiceDate   = new DateTime();

                            $currency = $originalInvoice->getCurrency();
                            $totalSum = new DecimalType($originalInvoice->getTotalSum());

                            $customerId                = new IdType($originalInvoice->getCustomerId());
                            $customerStatusId          = new IdType($originalInvoice->getCustomerStatusId());
                            $customerStatusName        = new StringType($originalInvoice->getCustomerName());
                            $customerStatusImage       = new StringType('');
                            $customerStatusDiscount    = new DecimalType(0.0);
                            $customerIsGuest           = new BoolType(true);
                            $customerStatusInformation = MainFactory::create(
                                'CustomerStatusInformation',
                                $customerStatusId,
                                $customerStatusName,
                                $customerStatusImage,
                                $customerStatusDiscount,
                                $customerIsGuest
                            );

                            $customerServiceFactory = MainFactory::create(
                                'CustomerServiceFactory',
                                StaticGXCoreLoader::getDatabaseQueryBuilder()
                            );
                            $countryReadService     = $customerServiceFactory->getCountryService();

                            $invoicePaymentAddress = $originalInvoice->getPaymentAddress();
                            $paymentAddress        = new AddressBlock(
                                new CustomerGender(''),
                                new CustomerFirstname($invoicePaymentAddress->getFirstName()),
                                new CustomerLastname($invoicePaymentAddress->getLastName()),
                                new CustomerCompany($invoicePaymentAddress->getCompany()),
                                new CustomerB2BStatus(false),
                                new CustomerStreet($invoicePaymentAddress->getStreet()),
                                new CustomerHouseNumber($invoicePaymentAddress->getHouseNumber()),
                                new CustomerAdditionalAddressInfo($invoicePaymentAddress->getAdditionalAddressInfo()),
                                new CustomerSuburb(''),
                                new CustomerPostcode($invoicePaymentAddress->getPostcode()),
                                new CustomerCity($invoicePaymentAddress->getCity()),
                                $countryReadService->getCountryByName($invoicePaymentAddress->getCountry()),
                                $countryReadService->getUnknownCountryZoneByName('')
                            );

                            $invoiceShippingAddress = $originalInvoice->getShippingAddress();
                            $shippingAddress        = new AddressBlock(
                                new CustomerGender(''),
                                new CustomerFirstname($invoiceShippingAddress->getFirstName()),
                                new CustomerLastname($invoiceShippingAddress->getLastName()),
                                new CustomerCompany($invoiceShippingAddress->getCompany()),
                                new CustomerB2BStatus(false),
                                new CustomerStreet($invoiceShippingAddress->getStreet()),
                                new CustomerHouseNumber($invoiceShippingAddress->getHouseNumber()),
                                new CustomerAdditionalAddressInfo($invoiceShippingAddress->getAdditionalAddressInfo()),
                                new CustomerSuburb(''),
                                new CustomerPostcode($invoiceShippingAddress->getPostcode()),
                                new CustomerCity($invoiceShippingAddress->getCity()),
                                $countryReadService->getCountryByName($invoiceShippingAddress->getCountry()),
                                $countryReadService->getUnknownCountryZoneByName('')
                            );

                            $orderPurchaseDate = $originalInvoice->getOrderDatePurchased();
                            $paymentType       = $originalInvoice->getPaymentType();
                        }

                        if ($cancelInvoice || (int)$orderId !== 0) {
                            $invoiceInfo = MainFactory::create(
                                'InvoiceInformation',
                                $invoiceNumber,
                                $invoiceDate,
                                $currency,
                                $totalSum,
                                $customerId,
                                $customerStatusInformation,
                                $paymentAddress,
                                $shippingAddress,
                                $orderId,
                                $orderPurchaseDate,
                                $paymentType
                            );

                            /* @var InvoiceArchiveWriteService $invoiceArchiveWriteService */
                            $invoiceArchiveWriteService = StaticGXCoreLoader::getService('InvoiceArchiveWrite');
                            // save the file reference to the invoice database entry
                            // attention - this is renaiming the original file name defined in path!!!
                            $invoiceId                  = $invoiceArchiveWriteService->importInvoiceFile($invoiceFile, $invoiceInfo);
                            // add to created ids to be able sned out new created invoices via email
                            array_push($createdInvoiceIds, $invoiceId);

                            @unlink($t_pdf_path);
                        }
                        $invoiceNumber = $invoiceInfo->getInvoiceNumber();
                    } else {
                        throw new Exception("lexoffice Invoice could not be created.");
                    }
                }
            } catch (Exception $e) {
                $this->logger->error('Error creating lexoffice invoice: ' . $e);
                $title    = new NonEmptyStringType($this->languageTextManager->get_text('lexoffice_invoice_api_error'));
                $template = $this->getTemplateFile('Agiltech/LexofficeInvoice/Admin/Html/lexoffice_invoice_error.html');

                $data = MainFactory::create(
                    'KeyValueCollection',
                    [
                        'error'            => $e->getMessage(),
                    ]
                );
                return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data);
            }
        }

        /**
         * -> handle output
         */
        if ($_GET['mail'] == 1) {
            require_once(DIR_FS_INC . 'xtc_php_mail.inc.php');

            /**
             * -> parse email text
             */
            $smarty = new Smarty;
            $smarty->setCompileDir(DIR_FS_CATALOG . StaticGXCoreLoader::getThemeControl()->getCompiledTemplatesFolder());

            $gm_logo_mail = MainFactory::create_object('GMLogoManager', array("gm_logo_mail"));
            if ($gm_logo_mail->logo_use == '1') {
                $smarty->assign('gm_logo_mail', $gm_logo_mail->get_logo());
            }

            if ($order_check['customers_gender'] == 'f') {
                $salutationFemale = gm_get_content('GM_PDF_SALUTATION_FEMALE', $_SESSION['languages_id']);
                $smarty->assign('SALUTATION', $salutationFemale);
            } else {
                $salutationMale = gm_get_content('GM_PDF_SALUTATION_MALE', $_SESSION['languages_id']);
                $smarty->assign('SALUTATION', $salutationMale);
            }

            $smarty->assign('CUSTOMER_GENDER', $order_check['customers_gender']);

            $customerName = '';

            if (isset($order_check['customers_firstname'])) {
                $customerName = $order_check['customers_firstname'] . ' ' . $order_check['customers_lastname'];
            } elseif (isset($invoiceListItem)) {
                $customerName = $invoiceListItem->getCustomerName();
            }

            $smarty->assign('CUSTOMER', $customerName);
            $smarty->assign('ORDER_ID', $orderId);
            $smarty->assign('INVOICE_ID', $order_check['gm_orders_code']);

            if (empty($order->info['date_purchased']) && isset($invoiceListItem)) {
                $smarty->assign('DATE', xtc_date_short($invoiceListItem->getOrderDatePurchased()->format('Y-m-d H:i:s')));
            } else {
                $smarty->assign('DATE', xtc_date_short($order->info['date_purchased']));
            }

            if (defined('EMAIL_SIGNATURE')) {
                $smarty->assign('EMAIL_SIGNATURE_HTML', nl2br(EMAIL_SIGNATURE));
                $smarty->assign('EMAIL_SIGNATURE_TEXT', EMAIL_SIGNATURE);
            }

            $t_mail_attachment_array = [];
            $invoiceNumbers = [];

            $invoiceIds = [];

            if (array_key_exists('invoice_ids', $_POST) && !empty($_POST['invoice_ids'])) {
                $invoiceIds = $_POST['invoice_ids'];
            } elseif (!empty($_GET['iID'])) {
                $invoiceIds[] = (int)$_GET['iID'];
            }

            // if new created invoices - also add to id list to send out via email
            if (!empty($createdInvoiceIds)) {
                $invoiceIds = array_merge($invoiceIds,  $createdInvoiceIds);
            }

            if (!empty($_GET['bulk'])) {
                $invoices = $invoiceArchiveReadService->getInvoiceListByConditions(['order_id' => (int)$orderId], new IntType(0), new IntType(1), new StringType('invoice_date DESC'));
                if (!$invoices->isEmpty()) {
                    /** @var InvoiceListItem $invoice */
                    $invoice = $invoices->getItem(0);

                    $invoiceNumbers[] = [
                        'number' => $invoice->getInvoiceNumber(),
                        'date' => xtc_date_short($invoice->getInvoiceDate()->format('Y-m-d H:i:s')),
                    ];

                    $t_pdf_name = xtc_cleanName(trim($gm_order_pdf_values['GM_PDF_HEADING']), '_') . '_' . $invoice->getInvoiceNumber() . '.pdf';
                    $t_pdf_path = DIR_FS_CATALOG . 'export/invoice/' . $invoice->getInvoiceFilename();

                    $t_mail_attachment_array[] = [
                        'name' => $t_pdf_name,
                        'path' => $t_pdf_path
                    ];

                    $smarty->assign('INVOICE_ID', $invoice->getInvoiceNumber());

                    $subject = str_replace('{DATE}', xtc_date_short($order->info['date_purchased']), $_POST['gm_subject']);
                    $subject = str_replace('{INVOICE_ID}', $invoice->getInvoiceNumber(), $subject);
                    $subject = str_replace('{ORDER_ID}', (int)$_GET['oID'], $subject);
                } else {
                    throw new Exception('Order ' . (int)$_GET['oID'] . ' has no invoice. No invoice mail was sent.');
                }
            } elseif (count($invoiceIds)) {
                foreach ($invoiceIds as $invoiceId) {
                    $invoices = $invoiceArchiveReadService->getInvoiceListByConditions(['invoice_id' => $invoiceId]);
                    if (!$invoices->isEmpty()) {
                        /** @var InvoiceListItem $invoice */
                        $invoice = $invoices->getItem(0);

                        $invoiceNumbers[] = [
                            'number' => $invoice->getInvoiceNumber(),
                            'date' => xtc_date_short($invoice->getInvoiceDate()->format('Y-m-d H:i:s')),
                        ];



                        $t_pdf_name = xtc_cleanName(trim($gm_order_pdf_values['GM_PDF_HEADING']), '_') . '_' . $invoice->getInvoiceNumber() . '.pdf';
                        $t_pdf_path = DIR_FS_CATALOG . 'export/invoice/' . $invoice->getInvoiceFilename();

                        $t_mail_attachment_array[] = [
                            'name' => $t_pdf_name,
                            'path' => $t_pdf_path
                        ];

                        $subject = str_replace('{INVOICE_ID}', $invoice->getInvoiceNumber(), $_POST['gm_subject']);
                    }
                }
            } else {
                // no invoice created yet - cannot happen - missing invoices created above and added to id list!
            }

            $smarty->assign('INVOICE_NUMBERS', $invoiceNumbers);

            $smarty->caching = 0;
            $t_html_mail = fetch_email_template($smarty, 'invoice_mail');
            $mail_text = fetch_email_template($smarty, 'invoice_mail', 'txt');

            if (empty($_GET['gm_quick_mail']) == false) {
                $order_check['customers_email_address'] = $_POST['gm_mail'];
                $order_check['customers_firstname'] = '';
                $order_check['customers_lastname'] = '';
            }

            if (!filter_var($order_check['customers_email_address'], FILTER_VALIDATE_EMAIL)) {
                throw new Exception('Invalid email address provided: ', $order_check['customers_email_address']);
            }

            // Send Email
            $mailResult = xtc_php_mail(
                EMAIL_FROM,
                STORE_NAME,
                $order_check['customers_email_address'],
                $order_check['customers_firstname'] . ' ' . $order_check['customers_lastname'],
                EMAIL_BILLING_FORWARDING_STRING,
                '',
                '',
                $t_mail_attachment_array,
                '',
                $subject,
                $t_html_mail,
                strip_tags($mail_text)
            );
            if ($mailResult) {
                $successMessage = ' ' . lexoffice_invoice_pdf_mail_success . '<br><br><span class="btn pull-right" onclick="gm_mail_close(\'INVOICE_MAIL\')" style="cursor:pointer">' . lexoffice_invoice_pdf_mail_close . '</span>';
                return MainFactory::create('JsonHttpControllerResponse', ['success' => $successMessage]);
            }
        } elseif (array_key_exists('ajax', $_GET)) {
            $pathArray = explode('/', $t_pdf_path);
            $unpreparedFilename = $pathArray[count($pathArray) - 1];

            $fileNameArray  = explode('__', $unpreparedFilename);
            $fileNameSuffix = $fileNameArray[count($fileNameArray) - 1];
            $fileName       = str_replace('__' . $fileNameSuffix, '', $unpreparedFilename);
        } elseif (array_key_exists('download', $_GET)) {
            // nothing todo
        } elseif (isset($_GET['preview'])) {
            // nothing todo
        } else {
            $redirect = true;
        }

        // Prepare AJAX JSON response.
        if (array_key_exists('ajax', $_GET)) {
            $response = [];

            if ($invoiceId) {
                $response['invoiceId'] = $invoiceId;
            }

            if ($invoiceNumber) {
                $response['invoiceNumber'] = $invoiceNumber;
            }

            if ($fileName) {
                $response['filename'] = $fileName;
            }

            if ($fileNameSuffix) {
                $response['filenameSuffix'] = $fileNameSuffix;
            }

            return MainFactory::create('JsonHttpControllerResponse', $response);
        }

        if ($redirect) {
            $url = 'request_port.php?module=OrderAdmin&action=showPdf&type=invoice&invoice_number=' . rawurlencode($invoiceNumber) . '&order_id=' . (int)$_GET['oID'];
            return new RedirectHttpControllerResponse(DIR_WS_ADMIN . $url);
        }
    }
    /**
     * private Helper function
     */

    private function getOrderCheckData()
    {
        $order_query_check = xtc_db_query("
        SELECT
            gm_packings_id,
            gm_orders_id,
            gm_packings_code,
            gm_orders_code,
            gm_packings_code,
            customers_email_address,
            customers_firstname,
            customers_lastname,
            customers_gender,
            customers_status,
            gm_cancel_date,
            orders_status
        FROM " .
            TABLE_ORDERS . "
        WHERE
            orders_id='" . (int)$_GET['oID'] . "'
    ");

        return xtc_db_fetch_array($order_query_check);
    }

    private function getOrderData()
    {
        $order_data = array();

        /*
         * -> order data
         */
        $order_query = xtc_db_query("
            SELECT
            op.products_id,
            op.orders_products_id,
            op.products_model,
            op.products_name,
            op.final_price,
            op.products_tax,
            op.products_quantity,
            opqu.quantity_unit_id,
            opqu.unit_name
            FROM " . TABLE_ORDERS_PRODUCTS . " op
            LEFT JOIN orders_products_quantity_units opqu USING (orders_products_id)
            WHERE
            op.orders_id='" . (int)$_GET['oID'] . "'
            ORDER BY
            op.orders_products_id ASC
        ");

        while ($order_data_values = xtc_db_fetch_array($order_query)) {
            $attributes_query = xtc_db_query("
                                                SELECT
                                                    products_options,
                                                    products_options_values,
                                                    price_prefix,
                                                    options_values_price
                                                FROM " .
                TABLE_ORDERS_PRODUCTS_ATTRIBUTES . "
                                                WHERE
                                                    orders_products_id='" . $order_data_values['orders_products_id'] . "'
                                                    AND orders_id='" . (int)$_GET['oID'] . "'
                                            ");



            $attributes_data = array();
            while ($attributes_data_values = xtc_db_fetch_array($attributes_query)) {
                $attributes_data[] = array(
                    xtc_get_attributes_model($order_data_values['products_id'], $attributes_data_values['products_options_values'], $attributes_data_values['products_options']),
                    $attributes_data_values['products_options'] . ': ' . $attributes_data_values['products_options_values'],
                );
            }

            // BOF GM_MOD GX-Customizer:
            require(DIR_FS_CATALOG . 'gm/modules/gm_gprint_admin_gm_pdf_order.php');

            # properties BOF
            $t_properties_query = xtc_db_query("SELECT
                                                        properties_name,
                                                        values_name
                                                    FROM orders_products_properties
                                                    WHERE orders_products_id='" . $order_data_values['orders_products_id'] . "'");
            while ($t_properties_array = xtc_db_fetch_array($t_properties_query)) {
                $attributes_data[] = array('', html_entity_decode_wrapper($t_properties_array['properties_name']) . ': ' . html_entity_decode_wrapper($t_properties_array['values_name']));
            }
            # properties EOF

            if ($order_data_values['products_quantity'] == 0) {
                $t_products_price_single = xtc_format_price_order(0.0, 1, $order->info['currency']);
                $t_products_price_single = 0.0;
            } else {
                $t_products_price_single = xtc_format_price_order($order_data_values['final_price'] / $order_data_values['products_quantity'], 1, $order->info['currency']);
                $t_products_price_single = $order_data_values['final_price'] / $order_data_values['products_quantity'];
            }
            $order_data[] = array(
                'PRODUCTS_MODEL' => $order_data_values['products_model'],
                'PRODUCTS_NAME' => $order_data_values['products_name'],
                'PRODUCTS_QTY' => (float)gm_prepare_number($order_data_values['products_quantity']),
                'PRODUCTS_UNIT' => $order_data_values['unit_name'],
                'PRODUCTS_TAX' => (float)gm_prepare_number($order_data_values['products_tax']),
                'PRODUCTS_PRICE_SINGLE' => $t_products_price_single,
                'PRODUCTS_PRICE' => $order_data_values['final_price'],
                'PRODUCTS_ATTRIBUTES' => $attributes_data
            );
        }

        return $order_data;
    }

    private function getOrderTotalData()
    {
        $oder_total_query = xtc_db_query("
                                            SELECT
                                                  title,
                                                  text,
                                                  class,
                                                  value,
                                                  sort_order
                                            FROM " .
            TABLE_ORDERS_TOTAL . "
                                            WHERE
                                                orders_id='" . (int)$_GET['oID'] . "'
                                            ORDER BY
                                                sort_order ASC
                                            ");


        /*
         * -> order total data
         */
        $order_total = array();
        while ($oder_total_values = xtc_db_fetch_array($oder_total_query)) {
            $class = $oder_total_values['class'];
            switch ($class) {
                case 'ot_total':
                    $order_total['TOTAL_AMOUNT'] = $oder_total_values['value'];
                    break;

                case 'ot_tax':
                    $order_total['TOTAL_TAX'] = $oder_total_values['value'];
                    break;

                case 'ot_subtotal':
                case 'ot_subtotal_no_tax':
                case 'ot_total_netto':
                    // not relevant - do not create line item
                    break;

                default:
                    // for all other create optional line item using naming convention
                    $order_total['TOTAL_POS_' . strtoupper(substr($class, 3))] = array(
                        'TITLE' => html_entity_decode_wrapper($oder_total_values['title']),
                        'VALUE' => $oder_total_values['value']
                    );

                    break;
            }
        }
        return $order_total;
    }


    private function getOrderPaymentInstruction($paymantMethod)
    {
        $paymentInstruction = null;
        $query = 'SELECT * FROM `orders_payment_instruction` WHERE `orders_id` = \'' . (int)$_GET['oID'] . '\'';
        $result = xtc_db_query($query);
        while ($row = xtc_db_fetch_array($result)) {
            $paymentInstruction = $row;
        }
        if ($paymentInstruction !== null && $paymantMethod === 'paypal3' && $paymentInstruction['due_date'] !== '1000-01-01') {
            $paypalText = MainFactory::create('PayPalText');
            $paymentInstruction['additional_note'] = COMPANY_NAME . ' ' . $paypalText->get_text('payment_instruction_additional_note');
        }
        return $paymentInstruction;
    }

    private function changeOrderStatus($gmFormat, $order_check, $cancelInvoice)
    {
        $t_change_status = true;

        $orderStatusExists = xtc_db_query("
                SELECT *
                FROM " . TABLE_ORDERS_STATUS . "
                WHERE orders_status_id = '" . (int)gm_get_conf('GM_PDF_ORDER_STATUS_INVOICE') . "'
            ")->num_rows > 0;

        $t_order_status_id = $orderStatusExists ? gm_get_conf('GM_PDF_ORDER_STATUS_INVOICE') : 149;
        $t_customer_notified = 0;
        $t_comment = lexoffice_invoice_invoicing_comment;
        if ($order_check['orders_status'] == $t_order_status_id) {
            $t_change_status = false;
        }

        if ($t_change_status && !$cancelInvoice) {
            $gmFormat->update_orders_status($_GET['oID'], $t_order_status_id, $t_customer_notified, $t_comment, $_SESSION['customer_id']);
        } else {
            $gmFormat->update_orders_status($_GET['oID'], $order_check['orders_status'], 0, lexoffice_invoice_invoicing_comment, $_SESSION['customer_id']);
        }
    }

    private function makePaymentInstructionText($paymentInstruction)
    {
        $textLines = array(
            $this->languageTextManager->get_text('lexoffice_invoice_payment_instruction'),
            $this->languageTextManager->get_text('lexoffice_invoice_payment_note'),
            $this->languageTextManager->get_text('lexoffice_invoice_amount') . ': ' . sprintf("%.2f %s", $paymentInstruction['value'], $paymentInstruction['currency']),
            $this->languageTextManager->get_text('lexoffice_invoice_iban') . ': ' . $paymentInstruction['iban'],
            $this->languageTextManager->get_text('lexoffice_invoice_bic') . ': ' . $paymentInstruction['bic'],
            $this->languageTextManager->get_text('lexoffice_invoice_account_holder') . ': ' . $paymentInstruction['account_holder'],
            $this->languageTextManager->get_text('lexoffice_invoice_bank_name') . ': ' . $paymentInstruction['bank_name'],
            $this->languageTextManager->get_text('lexoffice_invoice_reference') . ': ' . $paymentInstruction['reference'],
        );

        if ($paymentInstruction['due_date'] !== '0000-00-00' && $paymentInstruction['due_date'] !== '1000-01-01') {
            $textLines[] = $this->languageTextManager->get_text('lexoffice_invoice_due_date') . ': '
                . xtc_date_short($paymentInstruction['due_date'] . ' 00:00:00');
        }

        if (!empty($paymentInstruction['additional_note'])) {
            $textLines[] = $paymentInstruction['additional_note'];
        }
        $text = implode("\n", $textLines);
        return $text;
    }

    private function makeShippingMethodText($shippingClass)
    {
        $text = $shippingClass;
        if (!empty($shippingClass)) {

            if (strstr($shippingClass, '_')) {
                $gm_shipping_class = explode('_', $shippingClass);
                $gm_shipping = $gm_shipping_class[0];
            }

            $coo_lang_file_master = MainFactory::create_object('LanguageTextManager', array(), true);
            $coo_lang_file_master->init_from_lang_file('lang/' . $_SESSION['language'] . '/modules/shipping/' . $gm_shipping . '.php');
            if (defined(strtoupper('MODULE_SHIPPING_' . $gm_shipping . '_TEXT_TITLE'))) {
                $text = trim(html_entity_decode_wrapper(strip_tags(constant(strtoupper('MODULE_SHIPPING_' . $gm_shipping . '_TEXT_TITLE')))));
            } else {
                $text = $order_info['SHIPPING_METHOD'][1] = $gm_shipping;
            }
        }
        return $text;
    }

    private function prepareLexofficeInvoiceData($order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction)
    {
        $lxoInvoiceRequest = new LexofficeInvoiceInvoice();

        $this->setLexofficeVoucherData($lxoInvoiceRequest, $order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction);

        $lxoShippingCondition = new LexofficeShippingCondition();
        $lxoShippingCondition->setShippingType('none');
        // $lxoShippingCondition->setShippingDate($this->formatDateIso8601($t_invoice_date));
        $lxoInvoiceRequest->setShippingConditions($lxoShippingCondition);

        return $lxoInvoiceRequest;
    }

    private function prepareLexofficeCreditNoteData($order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction)
    {
        $lxoInvoiceRequest = new LexofficeInvoiceCreditNote();

        $this->setLexofficeVoucherData($lxoInvoiceRequest, $order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction);

        return $lxoInvoiceRequest;
    }

    private function setLexofficeVoucherData($lxoInvoiceRequest, $order, $order_data, $order_total, $order_check, $t_invoice_date, $order_payment_instruction)
    {
        $lxoInvoiceRequest->setVoucherDate($this->formatDateIso8601($t_invoice_date));

        $customer = $order->customer;
        $company = $customer['company'];
        $vatId = $customer['vat_id'];

        $taxType = $this->getLexofficeTaxType($this->configuration->getTaxType(), $order->delivery['country_iso_code_2'], $company, $vatId);

        $lxoLineItems = [];
        $productTax = null;
        foreach ($order_data as $order_item) {
            $productTax = round($order_item['PRODUCTS_TAX'], 4);

            $lxoLineItem = new LexofficeLineItem();
            $lxoLineItem->setType('custom');
            $lxoLineItem->setName($order_item['PRODUCTS_NAME']);

            $description = 'Artikel-Nr. ' . $order_item['PRODUCTS_MODEL'];
            foreach ($order_item['PRODUCTS_ATTRIBUTES'] as $attribute) {
                $description = $description . $attribute[0] . "\r\n" . '- ' . $attribute[1];
            }
            $lxoLineItem->setDescription($description);

            $lxoLineItem->setQuantity(round($order_item['PRODUCTS_QTY'], 4));
            $lxoUnitPrice = new LexofficeUnitPrice();
            $lxoUnitPrice->setCurrency('EUR');
            $divideValue = 100 + (float)$productTax;
            // round all amounts to precision of 4 digits
            $net         = round(($order_item['PRODUCTS_PRICE_SINGLE'] / $divideValue) * 100, 4);
            $lxoUnitPrice->setNetAmount($net); // depends on lxo taxtype (set gross or net amount)!
            $lxoUnitPrice->setGrossAmount(round($order_item['PRODUCTS_PRICE_SINGLE'], 4)); // depends on lxo taxtype (set gross or net amount)!

            if ($taxType === 'vatfree' || $taxType === 'intraCommunitySupply' || $taxType === 'thirdPartyCountryDelivery') {
                $lxoUnitPrice->setTaxRatePercentage(0);
            } else {
                $lxoUnitPrice->setTaxRatePercentage(round($productTax, 0));
            }

            $lxoLineItem->setUnitPrice($lxoUnitPrice);
            $unitName = $order_item['PRODUCTS_UNIT'];
            $unitName = !empty($unitName) ? $unitName : $this->languageTextManager->get_text('lexoffice_invoice_unit_name'); // fallback default translation
            $lxoLineItem->setUnitName(!empty($unitName) ? $unitName : ' - ');
            array_push($lxoLineItems, $lxoLineItem);
        }

        // set order total data 
        $lxoTotalPrice = new LexofficeTotalPrice();
        $lxoTotalPrice->setCurrency('EUR');
        $lxoTotalPrice->setTotalGrossAmount(round($order_total['TOTAL_AMOUNT'], 2));
        $lxoTotalPrice->setTotalTaxAmount(round($order_total['TOTAL_TAX'], 2));

        // iterate the order total additional positions to add a line item for each
        foreach ($order_total as $key => $value) {
            if (strpos($key, 'TOTAL_POS_') === 0) {
                if ($key == 'TOTAL_POS_COUPON' || $key == 'TOTAL_POS_DISCOUNT') {
                    // handle discounts
                    $lxoTotalPrice->setTotalDiscountAbsolute(round(abs($value['VALUE']), 2));
                } else {
                    $lxoLineItem = new LexofficeLineItem();
                    $lxoLineItem->setType('custom');
                    $lxoLineItem->setName($value['TITLE']);
                    $lxoLineItem->setQuantity(1);
                    $lxoUnitPrice = new LexofficeUnitPrice();
                    $lxoUnitPrice->setCurrency('EUR');
                    if ($key == 'TOTAL_POS_GV') {
                        // if voucher show as invoice item, but with negative price!
                        $lxoUnitPrice->setNetAmount(round($value['VALUE'], 2) * -1);
                        $lxoUnitPrice->setGrossAmount(round($value['VALUE'], 2) * -1);
                    } else {
                        $lxoUnitPrice->setNetAmount(round($value['VALUE'], 2));
                        $lxoUnitPrice->setGrossAmount(round($value['VALUE'], 2));
                    }
                    if ($key == 'TOTAL_POS_SHIPPING' && $taxType !== 'vatfree' && $taxType !== 'intraCommunitySupply' && $taxType !== 'thirdPartyCountryDelivery') {
                        $lxoUnitPrice->setTaxRatePercentage(round($productTax, 0));
                    } else {
                        $lxoUnitPrice->setTaxRatePercentage(0);
                    }
                    $lxoLineItem->setUnitPrice($lxoUnitPrice);
                    $lxoLineItem->setUnitName($this->languageTextManager->get_text('lexoffice_invoice_unit_name'));
                    array_push($lxoLineItems, $lxoLineItem);
                }
            }
        }
        $lxoInvoiceRequest->setLineItems($lxoLineItems);
        $lxoInvoiceRequest->setTotalPrice($lxoTotalPrice);

        // handle lexoffice contacts if activated
        $contactId = $this->handleLexofficeContact($order, $order_check);

        // define addresses
        $lxoAddress = new LexofficeAddress();
        if (!empty($contactId)) {
            $this->logger->notice('assign contact to invoice: ' . $contactId);
            $lxoAddress->setContactId($contactId);
        } else {
            $this->logger->notice('assign no contact to invoice - use Sammelkontakt');
            $lxoAddress->setName($order->billing['name']);
            $lxoAddress->setStreet($order->billing['street_address'] . ' ' . $order->billing['house_number']);
            $lxoAddress->setZip($order->billing['postcode']);
            $lxoAddress->setCity($order->billing['city']);
            $lxoAddress->setCountryCode($order->billing['country_iso_code_2']);
            $lxoAddress->setSupplement($order->billing['additional_address_info']);
        }
        $lxoInvoiceRequest->setAddress($lxoAddress);

        $lxoTaxCondition = new LexofficeTaxCondition();
        $lxoTaxCondition->setTaxType($taxType);
        $lxoInvoiceRequest->setTaxConditions($lxoTaxCondition);

        // check if use remark or payment terms
        $footerSetting = $this->configuration->getLexofficeFooter();

        $footerText = '';
        if ($footerSetting !== 0) {
            // set payment method as remark
            $footerText = $this->languageTextManager->get_text('lexoffice_invoice_payment_method') . ': ' . PaymentTitleProvider::getStrippedTagsTitle($order->info['payment_method']);
            // set shipping method as remark
            if (!empty($order->info['shipping_class'])) {
                $shippingMethodText = $this->makeShippingMethodText($order->info['shipping_class']);
                $footerText = $footerText . "\n" . $this->languageTextManager->get_text('lexoffice_invoice_shipping_method') . ': ' . $shippingMethodText;
            }
            // add shipping address
            $footerText = $footerText . "\n\n" . $this->languageTextManager->get_text('lexoffice_invoice_shipping_address') . ":\n" . strip_tags(xtc_address_format($order->delivery['format_id'], $order->delivery, 0, '', "\n"));
            // if payment instruction available set as remark
            if (!empty($order_payment_instruction)) {
                $paymentInstructionText = $this->makePaymentInstructionText($order_payment_instruction);
                $footerText = $footerText . "\n\n" . $paymentInstructionText;
            }
            if ($footerSetting === '1' && $lxoInvoiceRequest instanceof LexofficeInvoiceInvoice) {
                // set payment type as payment conditions
                $lxoPaymentCondition = new LexofficePaymentCondition();
                $lxoPaymentCondition->setPaymentTermLabel($footerText);
                $lxoPaymentCondition->setPaymentTermDuration(30);
                $lxoInvoiceRequest->setPaymentConditions($lxoPaymentCondition);
            } else if ($footerSetting === '2') {
                $lxoInvoiceRequest->setRemark($footerText);
            } else {
                // use default so do not add anything
            }
        }
    }

    private function handleLexofficeContact($order, $order_check)
    {
        $lxoContactId = null;
        // check contact settings and add lexoffice contact if required
        $contactSetting = $this->configuration->getLexofficeContact();
        $contactGuestSetting = $this->configuration->getLexofficeGuestContact();
        $customerStatus = $order_check['customers_status'];
        $customer = $order->customer;
        $company = $customer['company'];

        if (($contactSetting === '1' && $customerStatus !== '1')
            || ($contactGuestSetting === '1' && $customerStatus === '1')
            || (!empty($company))
        ) {
            $this->logger->notice('add lexoffice contact to invoice');
            // contact activated - add lexoffice contact to invoice 
            $customerServiceFactory = MainFactory::create(
                'CustomerServiceFactory',
                StaticGXCoreLoader::getDatabaseQueryBuilder()
            );
            $customerReadService = $customerServiceFactory->createCustomerReadService();
            $customerWriteService = $customerServiceFactory->createCustomerWriteService();

            // check if contact already exists in lexoffice - therefor always load customer, if old order and customernumber does not already exists there
            $filterCustomerId = !empty($order->customer['ID']) ?  $order->customer['ID'] :  $order->customer['csID'];
            $this->logger->notice('$filterCustomerId: ' . $filterCustomerId);
            $chckCustomers = $customerReadService->filterCustomers(['customers_id' => $filterCustomerId]);
            $chckCustomer = array_shift($chckCustomers);
            $existingLxoContact = null;

            if (!empty($chckCustomer)) {
                $customerNumber = (string)$chckCustomer->getCustomerNumber()->__toString();
                $customerEmail = (string)$chckCustomer->getEmail()->__toString();
                $this->logger->notice('$customerEmail: ' . print_r($customerEmail, true));
                $existingLxoContact = $this->lexofficeService->findLexofficeContactByNumber(intval($customerNumber))[0];
                if (empty($existingLxoContact)) {
                    // if not found with customer number - try to find by email address
                    $existingLxoContact = $this->lexofficeService->findLexofficeContactByEmail($customerEmail)[0];
                }
            }

            if (!empty($existingLxoContact) && $existingLxoContact->id) {
                $this->logger->notice('assign existing contact');
                // if exists assign lxo voucher to existing contact
                $lxoContactId = $existingLxoContact->id;
            } else {
                $this->logger->notice('create new contact');
                // first of all create a contact in lexoffice for this gambio customer
                $lxoContactRequest = $this->prepareLexofficeContactData($order);
                $lxoContactResponse = $this->lexofficeService->createLexofficeContact($lxoContactRequest);
                // now read the generated customer to geth the generated customer number
                $lxoContact = $this->lexofficeService->readLexofficeContact($lxoContactResponse->id);
                $lxoCustomerNumber = $lxoContact->roles->customer->number;
                $lxoContactId = $lxoContact->id;

                // update gambio customer csID

                // Fetch existing customer record.
                $customers = $customerReadService->filterCustomers(['customers_id' => $filterCustomerId]);
                $storedCustomer = array_shift($customers);
                if (!empty($storedCustomer)) {
                    $storedCustomer->setCustomerNumber(MainFactory::create('CustomerNumber', (string)$lxoCustomerNumber));
                    // Update record and respond to client.
                    $customerWriteService->updateCustomer($storedCustomer);

                    // update gambio orders customer csID
                    $orderWriteService           = StaticGXCoreLoader::getService('OrderWrite');
                    $orderReadService            = StaticGXCoreLoader::getService('OrderRead');
                    $storedOrder = $orderReadService->getOrderById(new IdType($_GET['oID']));
                    $storedOrder->setCustomerNumber(new StringType((string)$lxoCustomerNumber));
                    $orderWriteService->updateOrder($storedOrder);
                }
            }
        }

        return $lxoContactId;
    }

    private function prepareLexofficeContactData($order)
    {
        $lxoContactRequest = new LexofficeContact();
        $lxoContactRequest->setVersion(0);

        $lxoRole = new LexofficeRole();
        $lxoCustomer = new LexofficeCustomer();
        $lxoRole->setCustomer($lxoCustomer);
        $lxoContactRequest->setRoles($lxoRole);

        $customer = $order->customer;
        $company = $customer['company'];
        $vatId = $customer['vat_id'];
        if (!empty($company)) {
            // if company fill respective data
            $lxoCompany = new LexofficeCompany();
            $lxoCompany->setName($customer['company']);
            $lxoCompany->setAllowTaxFreeInvoices(true);

            if (!empty($vatId) && $this->isEuropeanVatId($vatId)) {
                // set vat id only for eu countries - non eu vat ids maybe blocked by lexoffice
                $lxoCompany->setVatRegistrationId(str_replace(' ', '', $vatId));
            }

            $lxoContactRequestPerson = new LexofficeContactPerson();
            if (!empty($customer['gender'])) {
                $lxoContactRequestPerson->setSalutation($customer['gender'] === 'm' ? 'Herr' : 'Frau');
            }
            $lxoContactRequestPerson->setFirstName($customer['firstname']);
            $lxoContactRequestPerson->setLastName($customer['lastname']);
            $lxoContactRequestPerson->setPrimary(true);

            $lxoContactRequest->setCompany($lxoCompany);
        } else {
            // no company so add person data
            $lxoPerson = new LexofficePerson();
            if (!empty($customer['gender'])) {
                $lxoPerson->setSalutation($customer['gender'] === 'm' ? 'Herr' : 'Frau');
            }
            $lxoPerson->setFirstName($customer['firstname']);
            $lxoPerson->setLastName($customer['lastname']);
            $lxoContactRequest->setPerson($lxoPerson);
        }

        // set billing address
        $lxoBillingAddress = new LexofficeContactAddress();
        $lxoBillingAddress->setStreet($customer['street_address'] . ' ' . $customer['house_number']);
        $lxoBillingAddress->setZip($customer['postcode']);
        $lxoBillingAddress->setCity($customer['city']);
        $lxoBillingAddress->setCountryCode($order->billing['country_iso_code_2']);
        $lxoBillingAddress->setSupplement($customer['additional_address_info']);

        $lxoContactRequestAddresses = new LexofficeContactAddresses();
        $lxoContactRequestAddresses->setBilling([$lxoBillingAddress]);
        $lxoContactRequest->setAddresses($lxoContactRequestAddresses);

        // set email business address
        $lxoContactRequestEmailAddress = $customer['email_address'];
        $lxoContactRequestEmailAddresses = new LexofficeEmailAddresses();
        $lxoContactRequestEmailAddresses->setBusiness([$lxoContactRequestEmailAddress]);
        $lxoContactRequest->setEmailAddresses($lxoContactRequestEmailAddresses);

        // set phone business address
        $lxoContactRequestPersonPhoneNumber = $customer['telephone'];
        if (!empty($lxoContactRequestPersonPhoneNumber)) {
            $lxoContactRequestPhoneNumbers = new LexofficePhoneNumbers();
            $lxoContactRequestPhoneNumbers->setBusiness([$lxoContactRequestPersonPhoneNumber]);
            $lxoContactRequest->setPhoneNumbers($lxoContactRequestPhoneNumbers);
        }

        return $lxoContactRequest;
    }

    private function getLexofficeTaxType($defaultTaxType, $countryCode, $company, $originalVatId)
    {
        $taxType = $defaultTaxType;
        $vatId = str_replace(' ', '', $originalVatId); // remove all whitespaces

        // if vatfree - nothing to check just return default
        if ($taxType === 'vatfree') {
            return $taxType;
        }

        // for private customers without company use original tax type
        if (empty($company) && empty($vatId)) {
            return $taxType;
        }

        // for germany we use the original tax type
        if (!empty($company) && !empty($vatId) && strpos($vatId, 'DE') === 0) {
            return $taxType;
        }

        // for european vat ids we use intraCommunitySupply
        if (!empty($company) && !empty($vatId) && $this->isEuropeanVatId($vatId)) {
            // vat is available but not german set special tax type
            $taxType = 'intraCommunitySupply';
        }

        // for international companies we use thirdPartyCountryDelivery 
        if (!empty($company) && !$this->isEuropeanCountryCode($countryCode)) {
            // company in non eu country
            $taxType = 'thirdPartyCountryDelivery';
        }

        // for all others return default
        return $taxType;
    }

    private function isEuropeanVatId($originalVatId)
    {
        $vatId = str_replace(' ', '', $originalVatId); // remove all whitespaces

        $euVatIdpattern = "/^((AT)?U[0-9]{8}|(BE)?0[0-9]{9}|(BG)?[0-9]{9,10}|(CY)?[0-9]{8}L|(CZ)?[0-9]{8,10}|(DE)?[0-9]{9}|(DK)?[0-9]{8}|(EE)?[0-9]{9}|
        (EL|GR)?[0-9]{9}|(ES)?[0-9A-Z][0-9]{7}[0-9A-Z]|(FI)?[0-9]{8}|(FR)?[0-9A-Z]{2}[0-9]{9}|(GB)?([0-9]{9}([0-9]{3})?|[A-Z]{2}[0-9]{3})|
        (HU)?[0-9]{8}|(IE)?[0-9]S[0-9]{5}L|(IT)?[0-9]{11}|(LT)?([0-9]{9}|[0-9]{12})|(LU)?[0-9]{8}|(LV)?[0-9]{11}|(MT)?[0-9]{8}|
        (NL)?[0-9]{9}B[0-9]{2}|(PL)?[0-9]{10}|(PT)?[0-9]{9}|(RO)?[0-9]{2,10}|(SE)?[0-9]{12}|(SI)?[0-9]{8}|(SK)?[0-9]{10})$/";

        return preg_match($euVatIdpattern, $vatId);
    }

    private function isEuropeanCountryCode($countryCode)
    {
        $euCountryPattern = "/^((AT)|(BE)|(BG)|(CY)|(CZ)|(DE)|(DK)|(EE)|(EL|GR)?|(ES)|(FI)|(FR)|(GB)?|(HU)|(IE)|(IT)|(LT)|(LU)|(LV)|(MT)|(NL)|(PL)|(PT)|(RO)|(SE)|(SI)(SK))$/";

        return preg_match($euCountryPattern, $countryCode);
    }


    private function formatDateIso8601($invoiceDate, $midnight = true)
    {
        $formattedDate = null;
        $lxoInvoiceDate = clone $invoiceDate;

        if ($midnight === true) {
            date_time_set($lxoInvoiceDate, 0, 0, 0, 0);
            // do not use microtime() caus we wand milliseconds to 000
            $formattedDate = substr_replace($lxoInvoiceDate->format('c'), substr("0.00000000", 1, 4), 19, 0);
        } else {
            $formattedDate = substr_replace($lxoInvoiceDate->format('c'), substr(microtime(), 1, 4), 19, 0);
        }

        return $formattedDate;
    }
}
