<?php
/* --------------------------------------------------------------
   LexofficeInvoiceModuleCenterModule.inc.php 2021-02-17
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

class LexofficeInvoiceModuleCenterModule extends AbstractModuleCenterModule
{
    var $version = "1.1.2";

    protected function _init()
    {
        $this->title       = sprintf($this->languageTextManager->get_text('lexoffice_invoice_title'), $this->version);
        $this->description = $this->languageTextManager->get_text('lexoffice_invoice_description');
        $this->sortOrder   = 10000;
    }


    /**
     * Installs the module
     */
    public function install()
    {
        parent::install();

        /**
         * Rename the PdfCreator directory to deactivate the gambio default invoic pdf generation module
         * when lexoffice invoid module will be installed no othe invoiving module should be active
         * to avoid bookkeeping issues with generating different invoices
         */
        if (file_exists(DIR_FS_CATALOG . 'PdfCreator')) {
            // rename(DIR_FS_CATALOG . 'PdfCreator', DIR_FS_CATALOG . 'PdfCreator_deactivated');
            // according to php bug #54097	we use exec mv instead of php rename (https://bugs.php.net/bug.php?id=54097)
            exec("mv " . escapeshellarg(DIR_FS_CATALOG . 'PdfCreator') . " " . escapeshellarg(DIR_FS_CATALOG . 'PdfCreator_deactivated'));
        }

        /**
         * Rename the HTML Content folder from uninstalled
         * This is because files will overloaded because exists on file system
         * even if the module is not installed so rename on installation that files will start overloading
         */
        if (file_exists(DIR_FS_CATALOG . 'GXModules/Agiltech/LexofficeInvoice/Admin/Html/content_deactivated')) {
            // rename(DIR_FS_CATALOG . 'PdfCreator', DIR_FS_CATALOG . 'PdfCreator_deactivated');
            // according to php bug #54097	we use exec mv instead of php rename (https://bugs.php.net/bug.php?id=54097)
            exec("mv " . escapeshellarg(DIR_FS_CATALOG . 'GXModules/Agiltech/LexofficeInvoice/Admin/Html/content_deactivated') . " " . escapeshellarg(DIR_FS_CATALOG . 'GXModules/Agiltech/LexofficeInvoice/Admin/Html/content'));
        }

        // preset the trial version license key bo be able to immediately start trial period without manual enter the key
        $this->storage()->setLicenseKey('TRIAL-DMNC-YFU9-ZU25-T4V3-P1GS-P2FE');
    }

    /**
     * Uninstall module
     */
    public function uninstall()
    {
        parent::uninstall();

        /**  
         * Rename the PdfCreator directory to deactivate the gambio default invoic pdf generation module
         * when lexoffice invoid module will be installed no othe invoiving module should be active
         * to avoid bookkeeping issues with generating different invoices
         */
        if (file_exists(DIR_FS_CATALOG . 'PdfCreator_deactivated')) {
            // rename(DIR_FS_CATALOG . 'PdfCreator_deactivated', DIR_FS_CATALOG . 'PdfCreator');
            // according to php bug #54097	we use exec mv instead of php rename (https://bugs.php.net/bug.php?id=54097)
            exec("mv " . escapeshellarg(DIR_FS_CATALOG . 'PdfCreator_deactivated') . " " . escapeshellarg(DIR_FS_CATALOG . 'PdfCreator'));
        }

        /**
         * Rename the HTML Content folder to uninstalled
         * This is because othervice thos files are still overloaded because exists on file system
         * even if the module was uninstalled
         */
        if (file_exists(DIR_FS_CATALOG . 'GXModules/Agiltech/LexofficeInvoice/Admin/Html/content')) {
            // rename(DIR_FS_CATALOG . 'PdfCreator', DIR_FS_CATALOG . 'PdfCreator_deactivated');
            // according to php bug #54097	we use exec mv instead of php rename (https://bugs.php.net/bug.php?id=54097)
            exec("mv " . escapeshellarg(DIR_FS_CATALOG . 'GXModules/Agiltech/LexofficeInvoice/Admin/Html/content') . " " . escapeshellarg(DIR_FS_CATALOG . 'GXModules/Agiltech/LexofficeInvoice/Admin/Html/content_deactivated'));
        }

        // deactivate license on uninstall
        try {
            $lexofficeInvoiceLicenseService = MainFactory::create('LexofficeInvoiceLicenseService', $this->languageTextManager);
            $lexofficeInvoiceLicenseService->deactivateLicense($this->storage()->getLicenseKey());
        } catch (LexofficeInvoiceLicenseException $exception) {
            $logger = new LexofficeInvoiceLogger();
            $logger->error('Could not deactivate license after unsinstall: ' . $exception->getMessage());
            $GLOBALS['messageStack']->add_session(
                sprintf($this->languageTextManager->get_text('lexoffice_invoice_error_deactivate_license'), $this->storage()->getLicenseKey()),
                'warning'
            );
        }

        $this->storage()->delete_all_trial();
    }

    /**
     * @return LexofficeInvoiceConfigurationStorage
     */
    protected function storage(): LexofficeInvoiceConfigurationStorage
    {
        if ($this->storage === null) {

            $this->storage = new LexofficeInvoiceConfigurationStorage;
        }

        return $this->storage;
    }
}
