<?php

/* --------------------------------------------------------------
   LexofficeInvoiceOrdersModalsAjaxController.inc.php 2020-09-11
   http://www.agiltech.de
   Copyright (c) 2020 agilTech Information Technologies GmbH
   --------------------------------------------------------------
*/

/**
 * Class LexofficeInvoiceOrdersModalsAjaxController
 *
 * This is a lexoffice overload for the OrdersModalsAjaxController.
 *
 * @see OrdersModalsAjaxController
 */
class LexofficeInvoiceOrdersModalsAjaxController extends LexofficeInvoiceOrdersModalsAjaxController_parent
{
	/**
	 * Overwrite Get Email-Invoice Subject
	 */
	public function actionGetEmailInvoiceSubject()
	{
		$configurationStorage = MainFactory::create('LexofficeInvoiceConfigurationStorage');

		if (gm_get_conf('MODULE_CENTER_LEXOFFICEINVOICE_INSTALLED') == true && $configurationStorage->hasLicensedAccessRights()) {
			/** @var InvoiceArchiveReadService $invoiceReader */
			$invoiceReader   = StaticGXCoreLoader::getService('InvoiceArchiveRead');
			$orderId         = $this->_getQueryParameter('id');
			$invoices        = $invoiceReader->getInvoiceListByConditions(['order_id' => $orderId]);
			$invoiceIdExists = $invoices->isEmpty();
			$invoiceNumbers  = [];
			$dateFormat      = DATE_FORMAT;
			$subject         = gm_get_content('GM_PDF_EMAIL_SUBJECT', $_SESSION['languages_id']);

			if ($invoices->count() === 1) {
				/** @var InvoiceListItem $invoice */
				$invoice = $invoices->getItem(0);

				$invoiceNumbers[$invoice->getInvoiceId()] = $invoice->getInvoiceNumber();

				$orderDate     = $invoice->getOrderDatePurchased();
				$invoiceNumber = $invoice->getInvoiceNumber();
			} elseif ($invoices->count() > 1) {
				/** @var InvoiceListItem $invoice */
				foreach ($invoices as $invoice) {
					$invoiceNumbers[$invoice->getInvoiceId()] = $invoice->getInvoiceNumber();
				}

				$subject = gm_get_content('GM_PDF_INVOICES_EMAIL_SUBJECT', $_SESSION['languages_id']);

				$orderDate     = $invoice->getOrderDatePurchased();
				$invoiceNumber = $invoice->getInvoiceNumber();
			} else {
				$orderDate     = new DateTime($this->_getQueryParameter('date'));
				$orderFormat   = new GMOrderFormat();
				$invoiceNumber = '{INVOICE_ID}';
			}

			$subject = str_replace(
				['{ORDER_ID}', '{DATE}', '{INVOICE_ID}', '{INVOICE_NUMBERS}'],
				[
					$orderId,
					$orderDate->format($dateFormat),
					$invoiceNumber,
					implode(', ', $invoiceNumbers)
				],
				$subject
			);

			// Return the response back to the client.
			return MainFactory::create(
				'JsonHttpControllerResponse',
				[
					'subject'         => $subject,
					'invoiceIdExists' => $invoiceIdExists,
					'invoiceNumbers'  => $invoiceNumbers
				]
			);
		} else {
			return parent::actionGetEmailInvoiceSubject();
		}
	}

	/**
	 * Cancel Order Callback
	 *
	 * This method uses the OrderActions class to cancel an order and fulfill the requirements of the cancellation
	 * (re-stock product, inform customer ...).
	 *
	 * @return JsonHttpControllerResponse
	 */
	public function actionCancelOrder()
	{
		$configurationStorage = MainFactory::create('LexofficeInvoiceConfigurationStorage');

		if (gm_get_conf('MODULE_CENTER_LEXOFFICEINVOICE_INSTALLED') == true && $configurationStorage->hasLicensedAccessRights()) {
			$orderActions = MainFactory::create('OrderActions');

			$orderIds                  = $this->_getPostData('selectedOrders');
			$restockQuantity           = new BoolType($this->_getPostData('reStock') === 'true');
			$recalculateShippingStatus = new BoolType($this->_getPostData('reShip') === 'true');
			$resetArticleStatus        = new BoolType($this->_getPostData('reActivate') === 'true');
			$notifyCustomer            = new BoolType($this->_getPostData('notifyCustomer') === 'true');
			$sendComment               = new BoolType($this->_getPostData('sendComments') === 'true');
			$comment                   = new StringType($this->_getPostData('cancellationComments'));

			$orderActions->cancelOrder(
				$orderIds,
				$restockQuantity,
				$recalculateShippingStatus,
				$resetArticleStatus,
				$notifyCustomer,
				$sendComment,
				$comment
			);

			$urls = [];

			if ($this->_getPostData('cancelInvoice') === 'true') {
				/** @var InvoiceArchiveReadService $invoiceArchiveReadService */
				$invoiceArchiveReadService = StaticGXCoreLoader::getService('InvoiceArchiveRead');

				foreach ($orderIds as $orderId) {
					$invoices = $invoiceArchiveReadService->getInvoiceListByConditions(
						['order_id' => $orderId],
						null,
						null,
						new StringType('invoice_date DESC')
					);

					if (!$invoices->isEmpty()) {
						/** @var InvoiceListItem $invoice */
						$invoice = $invoices->getItem(0);

						if (!$invoice->isCancellationInvoice()) {
							$urls[] = '/admin/admin.php?do=LexofficeInvoiceModule/CreateInvoice&oID=' . (int)$orderId
								. '&type=invoice&cancel_invoice_id='
								. $invoice->getInvoiceId();
						}
					}
				}
			}

			return MainFactory::create('JsonHttpControllerResponse', ['urls' => $urls]);
		} else {
			return parent::actionCancelOrder();
		}
	}
}
